import 'dart:convert';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:dio/dio.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:retgoo_sdk/models/user.dart';
import 'package:retgoo_sdk/protocol/api.dart';
import 'package:shared_preferences/shared_preferences.dart';

enum AuthenticationState {
  initializing,
  authenticated,
  unautenticated,
  dashboard,
}

class AuthenticationProvider with ChangeNotifier {
  AuthenticationState _state = AuthenticationState.initializing;
  AuthenticationState get state => _state;

  UserAccount _user;
  UserAccount get currentUser => _user;

  AuthenticationProvider() {
    FirebaseAuth.instance.authStateChanges().listen((firebaseUser) async {
      if (firebaseUser == null) {
        _state = AuthenticationState.unautenticated;
        notifyListeners();
        return;
      }

      final sharedPreference = await SharedPreferences.getInstance();
      if (sharedPreference.containsKey("token")) {
        accessToken = sharedPreference.getString("token");
        _state = AuthenticationState.authenticated;
        await refreshUser();
      } else {
        _state = AuthenticationState.dashboard;
        notifyListeners();
      }
    });

    if (FirebaseAuth.instance.currentUser == null) {
      _state = AuthenticationState.unautenticated;
    }
  }

  Future<bool> setUser(UserAccount account) async {
    final sharedPreference = await SharedPreferences.getInstance();
    await sharedPreference.setString("token", account.accessToken);
    accessToken = account.accessToken;
    _state = AuthenticationState.authenticated;
    await refreshUser();
    return true;
  }

  Future<void> switchUser() async {
    final sp = await SharedPreferences.getInstance();
    await sp.remove("token");
    sp.reload();
    accessToken = null;
    _user = null;
    _state = AuthenticationState.dashboard;
    notifyListeners();
  }

  Future<UserAccount> login({
    String username,
    String password,
  }) async {
    String accessToken;

    final api = Dio(BaseOptions(
      responseType: ResponseType.json,
    ))
      ..interceptors.add(
        InterceptorsWrapper(
          onRequest: (request) {
            request.baseUrl = "http://$baseURL/api/";
            if (accessToken != null) {
              request.headers["Authorization"] = "Bearer $accessToken";
            }
          },
        ),
      );

    final result = await api.post("user/authentication", data: {
      "username": username,
      "password": password,
    });

    if (result.statusCode == 200) {
      if (result.data["success"]) {
        accessToken = result.data["token"];
        final _user = await _getUser(api);
        if (_user != null) {
          _user.accessToken = result.data["token"];

          final firestore = FirebaseFirestore.instance;
          final id = base64.encode(utf8.encode("$username@$baseURL"));
          await firestore
              .doc("users/${FirebaseAuth.instance.currentUser.uid}/portals/$id")
              .set({
            "nick": _user.fullName,
            "username": _user.username,
            "host": baseURL,
            "access_token": _user.accessToken,
          });

          return _user;
        }
      }
    }

    return null;
  }

  Future<void> logout() async {
    FirebaseAuth.instance.signOut();
  }

  Future<UserAccount> _getUser(Dio api) async {
    final response = await api.get("user/GetProfile");
    if (response.statusCode == 200) {
      final _user = UserAccount.fromJson(response.data);

      try {
        final bytes = base64.decode(_user.image48);
        _user.imageBytes = bytes;
        return _user;
      } catch (e) {}
    }

    return null;
  }

  Future<void> refreshUser({Dio xapi}) async {
    _user = await _getUser(xapi ?? api);
    notifyListeners();
  }
}
